@@ -18,7 +18,7 @@
  *
  * May, 2000. Implemented on FreeBSD 4.0-RELEASE (Compaq Armada M700)
  *
- $Id: battstat_applet.c,v 1.52 2003/06/25 16:40:31 mattk Exp $
+ $Id: battstat_applet.c,v 1.53.2.2 2003/10/26 01:11:29 kmaraas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -221,9 +221,15 @@ struct apm_power_info apminfo;
 struct apm_info apminfo;
 #endif
 
+#ifdef __linux__
+struct acpi_info acpiinfo;
+gboolean using_acpi;
+int acpi_count;
+#endif
+
 #ifdef __FreeBSD__
 void
-apm_readinfo (PanelApplet *applet)
+apm_readinfo (PanelApplet *applet, ProgressData * battstat)
 {
   /* This is how I read the information from the APM subsystem under
      FreeBSD.  Each time this functions is called (once every second)
@@ -242,7 +248,7 @@ apm_readinfo (PanelApplet *applet)
 }
 #elif __OpenBSD__
 void
-apm_readinfo (PanelApplet *applet)
+apm_readinfo (PanelApplet *applet, ProgressData * battstat)
 {
   /* Code for OpenBSD by Joe Ammond <jra@twinight.org>. Using the same
      procedure as for FreeBSD.
@@ -259,10 +265,27 @@ apm_readinfo (PanelApplet *applet)
 #elif __linux__
 
 // Declared in acpi-linux.c
-gboolean acpi_linux_read(struct apm_info *apminfo);
+gboolean acpi_linux_read(struct apm_info *apminfo, struct acpi_info *acpiinfo);
+
+gboolean acpi_callback (GIOChannel * chan, GIOCondition cond, gpointer data)
+{
+  ProgressData * battstat = (ProgressData *) data;
+
+  if (cond & (G_IO_ERR | G_IO_HUP)) {
+    acpi_linux_cleanup(&acpiinfo);
+    apminfo.battery_percentage = -1;
+    return FALSE;
+  }
+  
+  if (acpi_process_event(&acpiinfo)) {
+    acpi_linux_read(&apminfo, &acpiinfo);
+    pixmap_timeout(data);
+  }
+  return TRUE;
+}
 
 void
-apm_readinfo (PanelApplet *applet)
+apm_readinfo (PanelApplet *applet, ProgressData * battstat)
 {
   /* Code for Linux by Thomas Hood <jdthood@mail.com>. apm_read() will
      read from /proc/... instead and we do not need to open the device
@@ -270,14 +293,34 @@ apm_readinfo (PanelApplet *applet)
   */
   if (DEBUG) g_print("apm_readinfo() (Linux)\n");
 
-  // ACPI support added by Lennart Poettering <lennart@poettering.de> 10/27/2001
-  // First try ACPI kernel interface, than fall back on APM
-  if (!acpi_linux_read(&apminfo))
+  /* ACPI support added by Lennart Poettering <lennart@poettering.de> 10/27/2001
+   * Updated by David Moore <dcm@acm.org> 5/29/2003 to poll less and
+   *   use ACPI events. */
+  if (using_acpi && acpiinfo.event_fd >= 0) {
+    if (acpi_count <= 0) {
+      /* Only call this one out of 30 calls to apm_readinfo() (every 30 seconds)
+       * since reading the ACPI system takes CPU cycles. */
+      acpi_count=30;
+      acpi_linux_read(&apminfo, &acpiinfo);
+    }
+    acpi_count--;
+  }
+  /* If we lost the file descriptor with ACPI events, try to get it back. */
+  else if (using_acpi) {
+      if (acpi_linux_init(&acpiinfo)) {
+          battstat->acpiwatch = g_io_add_watch (acpiinfo.channel,
+              G_IO_IN | G_IO_ERR | G_IO_HUP,
+              acpi_callback, battstat);
+          acpi_linux_read(&apminfo, &acpiinfo);
+      }
+  }
+  else
     apm_read(&apminfo);
 }
+
 #else
 void
-apm_readinfo (PanelApplet *applet)
+apm_readinfo (PanelApplet *applet, ProgressData * battstat)
 {
   g_print("apm_readinfo() (Generic)\n");
   g_print(
@@ -342,7 +385,7 @@ pixmap_timeout( gpointer data )
       3 = Charging
   */
 
-   apm_readinfo (PANEL_APPLET (battery->applet));
+   apm_readinfo (PANEL_APPLET (battery->applet), battery);
    batterypresent = TRUE;
 #ifdef __FreeBSD__
    acline_status = apminfo.ai_acline ? 1 : 0;
@@ -675,6 +718,11 @@ destroy_applet (GtkWidget *widget, gpointer data)
    gtk_timeout_remove (pdata->pixtimer);
    pdata->pixtimer = 0;
    pdata->applet = NULL;
+   g_object_unref(G_OBJECT (pdata->pixgc));
+   g_object_unref(pdata->ac_tip);
+   g_object_unref(pdata->progress_tip);
+   g_object_unref(pdata->progressy_tip);
+   g_object_unref(pdata->testpixgc);
    gdk_pixmap_unref(pdata->pixmap);
    gdk_pixmap_unref(pdata->pixmapy);
    gdk_pixmap_unref(pdata->pixbuffer);
@@ -682,6 +730,14 @@ destroy_applet (GtkWidget *widget, gpointer data)
    gdk_bitmap_unref(pdata->masky);
    gdk_bitmap_unref(pdata->pixmask);
 
+#ifdef __linux__
+   if (using_acpi) {
+     g_source_remove(pdata->acpiwatch);
+     pdata->acpiwatch = 0;
+     acpi_linux_cleanup(&acpiinfo);
+   }
+#endif
+
    if (pdata->suspend_cmd)
    	g_free (pdata->suspend_cmd);
    
@@ -858,8 +914,8 @@ about_cb (BonoboUIComponent *uic,
 				strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				pixbuf);
    
-   if (pixbuf) 
-   	gdk_pixbuf_unref (pixbuf);
+   if (pixbuf)
+	g_object_unref (pixbuf);
 
    gtk_window_set_wmclass (GTK_WINDOW (about_box), "battery charge monitor", "Batter Charge Monitor");
    gtk_window_set_screen (GTK_WINDOW (about_box),
@@ -892,7 +948,7 @@ change_orient (PanelApplet       *applet,
    
    if (DEBUG) g_print("change_orient()\n");
 
-   apm_readinfo(PANEL_APPLET (applet));
+   apm_readinfo(PANEL_APPLET (applet), battstat);
 #ifdef __FreeBSD__
    acline_status = apminfo.ai_acline ? 1 : 0;
    batt_state = apminfo.ai_batt_stat;
@@ -1202,35 +1258,6 @@ adj_value_changed_cb (GtkAdjustment *ignored, gpointer data)
    battstat->colors_changed = FALSE;
 }
 
-#if 0
-static void
-build_our_plug(StatusDocklet *docklet, GtkWidget *plug, gpointer data)
-{
-   ProgressData *battstat = data;
-   
-   battstat->eventdock = gtk_event_box_new ();
-   gtk_widget_show (battstat->eventdock);
-   gtk_container_add (GTK_CONTAINER(plug), battstat->eventdock);
-   
-   battstat->pixmapdockwid = gtk_pixmap_new ( statusimage[BATTERY], statusmask[BATTERY] );
-   
-   if(battstat->usedock) {
-      gtk_widget_show(battstat->eventdock);
-      gtk_widget_show(battstat->pixmapdockwid);
-   } else {
-      gtk_widget_hide(battstat->pixmapdockwid);    
-      gtk_widget_hide(battstat->eventdock);    
-   }
-   
-   gtk_container_add(GTK_CONTAINER(battstat->eventdock), battstat->pixmapdockwid);
-   
-   battstat->st_tip = gtk_tooltips_new ();
-   gtk_tooltips_set_tip (battstat->st_tip,
-			 battstat->eventdock,
-			 "Add code here",
-			 NULL);
-}
-#endif
 
 void
 change_size(PanelApplet *applet, gint size, gpointer data)
@@ -1296,14 +1323,12 @@ load_preferences(ProgressData *battstat)
   battstat->fullbattnot = panel_applet_gconf_get_bool (applet, GCONF_PATH "full_battery_notification", NULL);
   battstat->beep = panel_applet_gconf_get_bool (applet, GCONF_PATH "beep", NULL);
   battstat->draintop = panel_applet_gconf_get_bool (applet, GCONF_PATH "drain_from_top", NULL);
-  battstat->horizont = panel_applet_gconf_get_bool (applet, GCONF_PATH "horizontal", NULL);
-
+  
   battstat->showstatus = panel_applet_gconf_get_bool (applet, GCONF_PATH "show_status", NULL);
   battstat->showbattery = panel_applet_gconf_get_bool (applet, GCONF_PATH "show_battery", NULL);
   battstat->showpercent = panel_applet_gconf_get_bool (applet, GCONF_PATH "show_percent", NULL);
   battstat->suspend_cmd = panel_applet_gconf_get_string (applet, GCONF_PATH "suspend_command", NULL);
-  battstat->usedock = panel_applet_gconf_get_bool (applet, GCONF_PATH "use_dock", NULL);
-
+  
 }
 
 gint
@@ -1455,28 +1480,26 @@ create_layout(ProgressData *battstat)
    
    /* Set the default tooltips.. */
    battstat->ac_tip = gtk_tooltips_new ();
+   g_object_ref (battstat->ac_tip);
+   gtk_object_sink (GTK_OBJECT (battstat->ac_tip));
    gtk_tooltips_set_tip (battstat->ac_tip,
 			 battstat->eventstatus,
 			 "",
 			 NULL);
    battstat->progress_tip = gtk_tooltips_new ();
+   g_object_ref (battstat->progress_tip);
+   gtk_object_sink (GTK_OBJECT (battstat->progress_tip));
    gtk_tooltips_set_tip (battstat->progress_tip,
 			 battstat->eventbattery,
 			 "",
 			 NULL);
    battstat->progressy_tip = gtk_tooltips_new ();
+   g_object_ref (battstat->progressy_tip);
+   gtk_object_sink (GTK_OBJECT (battstat->progressy_tip));
    gtk_tooltips_set_tip (battstat->progressy_tip,
 			 battstat->eventybattery,
 			 "",
 			 NULL);
-#if 0
-   statusdock = status_docklet_new();
-   
-   gtk_signal_connect(GTK_OBJECT(statusdock), "build_plug",
-		      GTK_SIGNAL_FUNC(build_our_plug),
-		      battstat);
-   status_docklet_run(STATUS_DOCKLET(statusdock));
-#endif
 
    gtk_signal_connect(GTK_OBJECT(battstat->applet),"change_orient",
 		      GTK_SIGNAL_FUNC(change_orient),
@@ -1496,6 +1519,7 @@ static gboolean
 battstat_applet_fill (PanelApplet *applet)
 {
   ProgressData *battstat;
+  struct stat statbuf;
 
   if (DEBUG) g_print("main()\n");
   
@@ -1504,8 +1528,29 @@ battstat_applet_fill (PanelApplet *applet)
   panel_applet_add_preferences (applet, "/schemas/apps/battstat-applet/prefs", NULL);
   panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 
-  apm_readinfo (applet);
-  
+#ifdef __linux__
+  if (acpi_linux_init(&acpiinfo)) {
+    using_acpi = TRUE;
+    acpi_count = 0;
+  }
+  else
+    using_acpi = FALSE;
+
+  /* If neither ACPI nor APM could be read, but ACPI does seem to be
+   * installed, warn the user how to get ACPI working. */
+  if (!using_acpi && (apm_exists() == 1) &&
+          (stat("/proc/acpi", &statbuf) == 0)) {
+      battstat_error_dialog (
+	       applet,
+	       "Can't access ACPI events in /var/run/acpid.socket!\n\n"
+		 "Make sure the ACPI subsystem is working and\n"
+		 "the acpid daemon is running.");
+      using_acpi = TRUE;
+      acpi_count = 0;
+  }
+#endif
+  apm_readinfo (applet, NULL);
+
 #ifdef __FreeBSD__
   if(apminfo.ai_status == 0) cleanup (applet, 2);
 #endif
@@ -1528,6 +1573,7 @@ battstat_applet_fill (PanelApplet *applet)
   battstat->suspend_cmd = NULL;
   battstat->orienttype = panel_applet_get_orient (applet);
   battstat->panelsize = panel_applet_get_size (applet);
+  battstat->horizont = TRUE;
   
   glade_gnome_init ();
   
@@ -1538,6 +1584,14 @@ battstat_applet_fill (PanelApplet *applet)
   change_orient (applet, battstat->orienttype, battstat );
 
   battstat->pixtimer = gtk_timeout_add (1000, pixmap_timeout, battstat);
+#ifdef __linux__
+  /* Watch for ACPI events and handle them immediately with acpi_callback(). */
+  if (using_acpi && acpiinfo.event_fd >= 0) {
+    battstat->acpiwatch = g_io_add_watch (acpiinfo.channel,
+        G_IO_IN | G_IO_ERR | G_IO_HUP,
+        acpi_callback, battstat);
+  }
+#endif
 
   gtk_container_add (GTK_CONTAINER (battstat->applet), battstat->hbox1);
 
