@@ -45,15 +45,15 @@
 #include "inlinepixbufs.h"
 
 #define TIMEOUT_VALUE 500
-#define CDPLAYER_STOP           "cdplayer-stop"
-#define CDPLAYER_PLAY           "cdplayer-play"
-#define CDPLAYER_PAUSE          "cdplayer-pause"
-#define CDPLAYER_PREV           "cdplayer-prev"
-#define CDPLAYER_NEXT           "cdplayer-next"
-#define CDPLAYER_EJECT          "cdplayer-eject"
+#define CDPLAYER_STOP           "media-stop"
+#define CDPLAYER_PLAY           "media-play"
+#define CDPLAYER_PAUSE          "media-pause"
+#define CDPLAYER_PREV           "media-prev"
+#define CDPLAYER_NEXT           "media-next"
+#define CDPLAYER_EJECT          "media-eject"
 
 /* Function prototypes */
-static gboolean applet_factory (PanelApplet *applet, const gchar *iid, gpointer data);
+static gboolean cdplayer_applet_factory (PanelApplet *applet, const gchar *iid, gpointer data);
 static gboolean applet_fill (PanelApplet *applet);
 static void cdplayer_load_config(CDPlayerData *cd);
 static void show_error(CDPlayerData *cd);
@@ -121,11 +121,11 @@ PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_CDPlayerApplet_Factory",
 			     PANEL_TYPE_APPLET,
                              "cdplayer",
                              "0",
-                              applet_factory,
+                              cdplayer_applet_factory,
                               NULL)
 
 static gboolean
-applet_factory (PanelApplet *applet,
+cdplayer_applet_factory (PanelApplet *applet,
         const gchar *iid,
         gpointer     data)
 {
@@ -303,6 +303,11 @@ cdplayer_destroy(GtkWidget * widget, gpointer data)
         g_object_set_data (G_OBJECT (cd->panel.applet), "tooltips", NULL);
     }
    
+    if (cd->play_image)
+        g_object_unref (cd->play_image);
+    if (cd->pause_image)
+        g_object_unref (cd->pause_image);
+
     if (cd->time_description)
         g_free(cd->time_description);
     if (cd->track_description)
@@ -484,6 +489,7 @@ preferences_cb (BonoboUIComponent *component,
     gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
     gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
     box = GTK_DIALOG(dialog)->vbox;
     
     hbox = gtk_hbox_new (FALSE, 12);
