@@ -25,6 +25,7 @@
 #include <string.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <panel-applet.h>
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-window-icon.h>
@@ -131,6 +132,44 @@ button_press_hack (GtkWidget      *widget,
     return FALSE;
 }
 
+/* Send button presses on the applet to the entry. This makes Fitts' law work (ie click on the bottom
+** most pixel and the key press will be sent to the entry */
+static gboolean
+send_button_to_entry_event (GtkWidget *widget, GdkEventButton *event, MCData *mc)
+{
+
+	if (event->button == 1) {
+		gtk_widget_grab_focus (mc->entry);
+		return TRUE;
+	}
+	return FALSE;
+
+}
+
+static gboolean
+key_press_cb (GtkWidget *widget, GdkEventKey *event, MCData *mc)
+{
+	switch (event->keyval) {	
+	case GDK_b:
+		if (event->state == GDK_CONTROL_MASK) {
+			mc_show_file_browser (NULL, mc);
+			return TRUE;
+		}
+		break;
+	case GDK_h:
+		if (event->state == GDK_CONTROL_MASK) {
+			mc_show_history (NULL, mc);
+			return TRUE;
+		}
+		break;
+	default:
+		break;
+	}
+
+	return FALSE;
+
+}
+
 void
 mc_applet_draw (MCData *mc)
 {
@@ -140,10 +179,14 @@ mc_applet_draw (MCData *mc)
     GtkWidget *frame;
     MCPreferences prefs = mc->preferences;
     int        size_frames = 0;
+    gchar     *command_text = NULL;
 
     if (mc->preferences.show_frame)
 	size_frames += 6;
 
+    if (mc->entry != NULL)
+	command_text = g_strdup (gtk_editable_get_chars (GTK_EDITABLE (mc->entry), 0, -1));
+
     mc->cmd_line_size_y = mc->preferences.normal_size_y - size_frames;   
 
     if (mc->applet_box) {
@@ -154,6 +197,11 @@ mc_applet_draw (MCData *mc)
 
     mc_create_command_entry (mc);
 
+    if (command_text != NULL) {
+	gtk_entry_set_text (GTK_ENTRY (mc->entry), command_text);
+	g_free (command_text);
+    }
+
     /* hbox for message label and buttons */
     if (prefs.normal_size_y > GNOME_Vertigo_PANEL_X_SMALL) 
 	hbox_buttons = gtk_vbox_new (TRUE, 0);
@@ -225,7 +273,7 @@ mc_applet_draw (MCData *mc)
         GtkWidget *hbox;
         
         hbox = gtk_hbox_new (FALSE, 2);
-        
+        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
 	if (mc->preferences.show_frame) {
 	   
   	    frame = gtk_frame_new (NULL);
@@ -300,6 +348,7 @@ mini_commander_applet_fill (PanelApplet *applet)
     mc->applet = applet;
 
     panel_applet_add_preferences (applet, "/schemas/apps/mini-commander/prefs", NULL);
+    panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
     mc_load_preferences (mc);
     command_line_init_stock_icons ();
 
@@ -314,6 +363,10 @@ mini_commander_applet_fill (PanelApplet *applet)
     gtk_widget_show (GTK_WIDGET (mc->applet));
     
     g_signal_connect (mc->applet, "destroy", G_CALLBACK (mc_destroyed), mc); 
+    g_signal_connect (mc->applet, "button_press_event",
+    			       G_CALLBACK (send_button_to_entry_event), mc);
+    g_signal_connect (mc->applet, "key_press_event",
+    			       G_CALLBACK (key_press_cb), mc);
 
     panel_applet_setup_menu_from_file (mc->applet,
 				       NULL,
@@ -325,8 +378,7 @@ mini_commander_applet_fill (PanelApplet *applet)
     set_atk_name_description (GTK_WIDGET (applet),
 			      _("Mini-Commander applet"),
 			      _("This applet adds a command line to the panel"));
-    gtk_tooltips_set_tip (mc->tooltips, GTK_WIDGET (applet),  _("Command Line"), NULL);
-
+    
     return TRUE;
 }
 
