@@ -46,7 +46,8 @@
 #include <libgnome/libgnome.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-window-icon.h>
-#include <panel-applet.h>
+#include <gconf/gconf-client.h>
+#include <panel-applet-gconf.h>
 #include <egg-screen-exec.h>
 #include <egg-screen-help.h>
 
@@ -188,6 +189,24 @@ long pv_buf[MAX_PV_BUF] = {
 };
 #endif
 
+static gboolean
+key_writable (PanelApplet *applet, const char *key)
+{
+	gboolean writable;
+	char *fullkey;
+	static GConfClient *client = NULL;
+	if (client == NULL)
+		client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	writable = gconf_client_key_is_writable (client, fullkey, NULL);
+
+	g_free (fullkey);
+
+	return writable;
+}
+
 #ifdef OSS_API
 static void
 get_channels (MixerData *data)
@@ -215,8 +234,9 @@ get_channels (MixerData *data)
 	if (!valid) {
 		ChannelData *cdata = data->channels->data;
 		data->mixerchannel = cdata->channel;
-		panel_applet_gconf_set_int (PANEL_APPLET (data->applet), "channel", 
-							  data->mixerchannel, NULL);
+		if (key_writable (PANEL_APPLET (data->applet), "channel"))
+			panel_applet_gconf_set_int (PANEL_APPLET (data->applet), "channel", 
+						    data->mixerchannel, NULL);
 	}
 }
 #endif
@@ -557,8 +577,12 @@ mixer_update_image (MixerData *data)
 	g_object_unref (scaled);	
 	g_object_unref (pixbuf);
 	
-	panel_applet_gconf_set_int (PANEL_APPLET (data->applet), "vol", data->vol);
-	panel_applet_gconf_set_bool (PANEL_APPLET (data->applet), "mute", data->mute);
+	if (key_writable (PANEL_APPLET (data->applet), "vol"))
+		panel_applet_gconf_set_int (PANEL_APPLET (data->applet), "vol", data->vol,
+					    NULL);
+	if (key_writable (PANEL_APPLET (data->applet), "mute"))
+		panel_applet_gconf_set_bool (PANEL_APPLET (data->applet), "mute", data->mute,
+					     NULL);
 			
 }
 
@@ -605,7 +629,7 @@ show_error_dialog (MixerData *data)
 	data->error_dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-					 ("Couldn't open mixer device %s\n"),
+					 _("Couldn't open mixer device %s\n"),
 					 data->device, NULL);
 
 	gtk_window_set_screen (GTK_WINDOW (data->error_dialog),
@@ -940,6 +964,8 @@ destroy_mixer_cb (GtkWidget *widget, MixerData *data)
 	}
 	if (data->error_dialog)
 		gtk_widget_destroy (data->error_dialog);
+	if (data->tooltips)
+		g_object_unref (data->tooltips);
 	g_free (data);
 }
 
@@ -1054,6 +1080,7 @@ mixer_about_cb (BonoboUIComponent *uic,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
                                  pixbuf);
 
+	g_object_unref (pixbuf);
 	gtk_window_set_screen (GTK_WINDOW (about),
 			       gtk_widget_get_screen (data->applet));
 	gtk_window_set_wmclass (GTK_WINDOW(about), "volume control", "Volume Control");
@@ -1205,6 +1232,7 @@ mixer_pref_cb (BonoboUIComponent *uic,
         data->prefdialog = dialog;
                            
 	vbox = GTK_DIALOG (dialog)->vbox;
+	gtk_box_set_spacing (GTK_BOX (vbox), 2);
 	
 	vbox1 = gtk_vbox_new (FALSE, 18);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 5);
@@ -1257,7 +1285,7 @@ mixer_pref_cb (BonoboUIComponent *uic,
         	GtkTreeIter iter;
         	
 		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, cdata->name, 1, cdata, -1);
+		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, _(cdata->name), 1, cdata, -1);
 		if (cdata->channel == data->mixerchannel) {
 			GtkTreePath *path;
 			path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
@@ -1265,7 +1293,12 @@ mixer_pref_cb (BonoboUIComponent *uic,
 		}
         	list = g_list_next (list);
         }
-        
+
+	if ( ! key_writable (PANEL_APPLET (data->applet), "channel")) {
+		gtk_widget_set_sensitive (tree, FALSE);
+		gtk_widget_set_sensitive (label, FALSE);
+	}
+
         g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree))), 
 			  	  "changed",
 			  	   G_CALLBACK (cb_row_selected), data);
@@ -1547,6 +1580,8 @@ mixer_applet_create (PanelApplet *applet)
 	mixer_load_volume_images (data);
 
         data->tooltips = gtk_tooltips_new ();                                   
+	g_object_ref (data->tooltips);
+	gtk_object_sink (GTK_OBJECT (data->tooltips));
         gtk_tooltips_set_tip (data->tooltips,
 			      data->applet,
 			      _(access_name),
