@@ -21,6 +21,8 @@
 #include <stickynotes_applet_callbacks.h>
 #include <stickynotes.h>
 
+#include <egg-screen-help.h>
+
 /* Applet Callback : Mouse button press on the applet. */
 gboolean applet_button_cb(GtkWidget *widget, GdkEventButton *event, StickyNotesApplet *applet)
 {
@@ -37,26 +39,18 @@ gboolean applet_button_cb(GtkWidget *widget, GdkEventButton *event, StickyNotesA
 
 	stickynotes_applet_update_icon(applet);
 	
-	return FALSE;
+	return TRUE;
 }
 
 /* Applet Callback : Keypress on the applet. */
 gboolean applet_key_cb(GtkWidget *widget, GdkEventKey *event, StickyNotesApplet *applet)
 {
 	if (event->type == GDK_KEY_PRESS && event->keyval == GDK_Return)
-		applet->pressed = TRUE;
-	
-	else if (event->type == GDK_KEY_RELEASE && event->keyval == GDK_Return) {
 		stickynotes_applet_do_default_action();	
-		applet->pressed = FALSE;
-	}
-	
 	else
 		return FALSE;
 	
-	stickynotes_applet_update_icon(applet);
-	
-	return FALSE;
+	return TRUE;
 }
 
 /* Applet Callback : Cross (enter or leave) the applet. */
@@ -146,32 +140,66 @@ void menu_destroy_all_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, cons
 /* Menu Callback : Show/Hide sticky notes */
 void menu_toggle_show_cb(BonoboUIComponent *uic, const gchar *path, Bonobo_UIComponent_EventType type, const gchar *state, StickyNotesApplet *applet)
 {
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/visible", strcmp(state, "0") != 0, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/visible", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/visible", strcmp(state, "0") != 0, NULL);
 }
 
 /* Menu Callback: Lock/Unlock sticky notes */
 void menu_toggle_lock_cb(BonoboUIComponent *uic, const gchar *path, Bonobo_UIComponent_EventType type, const gchar *state, StickyNotesApplet *applet)
 {
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/locked", strcmp(state, "0") != 0, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/locked", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/locked", strcmp(state, "0") != 0, NULL);
 }
 
 /* Menu Callback : Configure preferences */
 void menu_preferences_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, const gchar *verbname)
 {
 	stickynotes_applet_update_prefs();
+	gtk_window_set_screen(GTK_WINDOW(stickynotes->w_prefs), gtk_widget_get_screen(applet->w_applet));
 	gtk_window_present(GTK_WINDOW(stickynotes->w_prefs));
 }
 
 /* Menu Callback : Show help */
 void menu_help_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, const gchar *verbname)
 {
-    	gnome_help_display("stickynotes_applet", "stickynotes-introduction", NULL);
+	GError *error = NULL;
+	egg_help_display_on_screen("stickynotes_applet", "stickynotes-introduction", gtk_widget_get_screen(applet->w_applet), &error);
+	if (error) {
+		GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+							   _("There was an error displaying help: %s"), error->message);
+		g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
+		gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW(dialog), gtk_widget_get_screen(applet->w_applet));
+		gtk_widget_show(dialog);
+		g_error_free(error);
+	}
 }
 
 /* Menu Callback : Display About window */
 void menu_about_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, const gchar *verbname)
 {
-	gtk_window_present(GTK_WINDOW(stickynotes->w_about));
+	static const gchar *authors[] = {
+		"Loban A Rahman <loban@earthling.net>",
+		NULL
+	};
+
+	static const gchar *documenters[] = {
+		"Loban A Rahman <loban@earthling.net>",
+		NULL
+	};
+
+	const gchar *translator_credits = _("translator_credits");
+
+	GtkWidget *w_about = gnome_about_new(_("Sticky Notes"),
+					     VERSION,
+					     _("(c) 2002-2003 Loban A Rahman"),
+					     _("Sticky Notes for the Gnome Desktop Environment"),
+					     authors,
+					     documenters,
+					     strcmp(translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+					     stickynotes->icon_normal);
+
+	gtk_widget_show(w_about);
 }
 
 /* Preferences Callback : Save. */
@@ -185,13 +213,20 @@ void preferences_save_cb(gpointer data)
 	gboolean sticky = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stickynotes->w_prefs_sticky));
 	gboolean force_default = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stickynotes->w_prefs_force));
 
-	gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/width", width, NULL);
-	gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/height", height, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_color", sys_color, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_font", sys_font, NULL);
-	gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/settings/click_behavior", click_behavior, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/sticky", sticky, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/force_default", force_default, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/defaults/width", NULL))
+		gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/width", width, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/defaults/height", NULL))
+		gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/height", height, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/use_system_color", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_color", sys_color, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/use_system_font", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_font", sys_font, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/click_behavior", NULL))
+		gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/settings/click_behavior", click_behavior, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/sticky", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/sticky", sticky, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/force_default", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/force_default", force_default, NULL);
 }
 
 /* Preferences Callback : Change color. */
@@ -274,16 +309,28 @@ void preferences_apply_cb(GConfClient *client, guint cnxn_id, GConfEntry *entry,
 /* Preferences Callback : Response. */
 void preferences_response_cb(GtkDialog *dialog, gint response, gpointer data)
 {
-	if (response == GTK_RESPONSE_HELP)
-		gnome_help_display("stickynotes_applet", "stickynotes-introduction", NULL);
-	
-	else /* if (response == GTK_RESPONSE_CLOSE || response == GTK_RESPONSE_NONE) || GTK_RESPONSE_DELETE_EVENT */
+	if (response == GTK_RESPONSE_HELP) {
+		GError *error = NULL;
+		egg_help_display_on_screen("stickynotes_applet", "stickynotes-advanced-settings", gtk_widget_get_screen(GTK_WIDGET(dialog)), &error);
+		if (error) {
+			GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+								   _("There was an error displaying help: %s"), error->message);
+			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
+			gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
+			gtk_window_set_screen (GTK_WINDOW(dialog), gtk_widget_get_screen(GTK_WIDGET(dialog)));
+			gtk_widget_show(dialog);
+			g_error_free(error);
+		}
+	}
+
+	else
 		gtk_widget_hide(GTK_WIDGET(dialog));
 }
 
-/* About Callback : Response. */
-void about_response_cb(GtkDialog *dialog, gint response, gpointer data)
+/* Preferences Callback : Delete */
+gboolean preferences_delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-	if (response == GTK_RESPONSE_DELETE_EVENT || response == GTK_RESPONSE_OK || response == GTK_RESPONSE_CLOSE)
-		gtk_widget_hide(GTK_WIDGET(dialog));
+	gtk_widget_hide(widget);
+
+	return TRUE;
 }
