@@ -35,68 +35,71 @@
 #include <panel-applet-gconf.h>
 #include <glade/glade.h>
 
+#include <egg-screen-help.h>
+
 #define CFG_DEVICE "eth0"
 #define CFG_UPDATE_INTERVAL 2
 
 typedef enum {
-	BUSTED_LINK,
-	NO_LINK,
-	NONE,
-} AnimationState;
+	PIX_BROKEN,
+	PIX_NO_LINK,
+	PIX_SIGNAL_1,
+	PIX_SIGNAL_2,
+	PIX_SIGNAL_3,
+	PIX_SIGNAL_4,
+	PIX_NUMBER,
+} PixmapState;
+
+static char * pixmap_names[] = {
+	"broken-0.png",
+	"no-link-0.png",
+	"signal-1-40.png",
+	"signal-41-60.png",
+	"signal-61-80.png",
+	"signal-81-100.png",
+};
 
 typedef struct {
 	PanelApplet base;
 	gchar *device;
-	gboolean show_percent, show_dialogs;
+	gboolean show_percent;
 
 	GList *devices;
 
-	/* a glist of char*, pointing to available images */
-	GList *images;
-	/* a glist of char*, pointing to the no-link-XX images if any) */
-	GList *no_link_images;
-	/* a glist of char*, pointing to the broken-XX images (if any) */
-	GList *broken_images;
 	/* contains pointers into the images GList.
 	 * 0-100 are for link */
-	char *pixmaps[101];
+	GdkPixbuf*pixmaps[PIX_NUMBER];
 	/* pointer to the current used file name */
-	char *current_pixmap;
-	/* set to true when the applet is display animated connection loss */
-	gboolean flashing;
+	GdkPixbuf *current_pixbuf;
 
 	GtkWidget *pct_label;
 	GtkWidget *pixmap;
-	AnimationState state;
-	guint animate_timer;
+	GtkWidget *box;
 	guint timeout_handler_id;
 	FILE *file;
 	GtkTooltips *tips;
 	GtkWidget *prefs;
 } WirelessApplet;
 
-char *pixmap_extensions[] = 
-{
-	"png",
-	"xpm",
-	NULL
-};
-
 static GladeXML *xml = NULL;
 static gchar* glade_file=NULL;
 
 static void show_error_dialog (gchar*,...);
 static void show_warning_dialog (gchar*,...);
-static void show_message_dialog (gchar*,...);
 static int wireless_applet_timeout_handler (WirelessApplet *applet);
 static void wireless_applet_properties_dialog (BonoboUIComponent *uic,
 		WirelessApplet *applet);
+static void wireless_applet_help_cb (BonoboUIComponent *uic,
+		WirelessApplet *applet);
 static void wireless_applet_about_cb (BonoboUIComponent *uic,
 		WirelessApplet *applet);
+static void prefs_response_cb (GtkDialog *dialog, gint response, gpointer data);
 
 static const BonoboUIVerb wireless_menu_verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("WirelessProperties",
 			wireless_applet_properties_dialog),
+	BONOBO_UI_UNSAFE_VERB ("WirelessHelp",
+			wireless_applet_help_cb),
 	BONOBO_UI_UNSAFE_VERB ("WirelessAbout",
 			wireless_applet_about_cb),
 	BONOBO_UI_VERB_END
@@ -127,10 +130,7 @@ wireless_applet_draw (WirelessApplet *applet, int percent)
 {
 	const char *label_text;
 	char *tmp;
-
-	if (!GTK_WIDGET_REALIZED (applet->pixmap)) {
-		return;
-	}
+	PixmapState state;
 
 	/* Update the percentage */
 	if (percent > 0) {
@@ -150,77 +150,26 @@ wireless_applet_draw (WirelessApplet *applet, int percent)
 	g_free (tmp);
 
 	/* Update the image */
-	percent = CLAMP (percent, 0, 100);
+	percent = CLAMP (percent, -1, 100);
 
-	if (applet->pixmaps[percent] != applet->current_pixmap)
+	if (percent < 0)
+		state = PIX_BROKEN;
+	else if (percent == 0)
+		state = PIX_NO_LINK;
+	else if (percent <= 40)
+		state = PIX_SIGNAL_1;
+	else if (percent <= 60)
+		state = PIX_SIGNAL_2;
+	else if (percent <= 80)
+		state = PIX_SIGNAL_3;
+	else
+		state = PIX_SIGNAL_4;
+
+	if (applet->pixmaps[state] != applet->current_pixbuf)
 	{
-		applet->current_pixmap = applet->pixmaps[percent];
-		if ( !applet->flashing)
-		{
-			gtk_image_set_from_file
-				(GTK_IMAGE (applet->pixmap),
-				 applet->current_pixmap);
-		}
-	}
-}
-
-
-static gboolean
-wireless_applet_animate_timeout (WirelessApplet *applet) 
-{	
-	static int num = 0;
-	GList *image;
-	GList *animation_list = NULL;
-
-	switch (applet->state) {
-	case NO_LINK:
-		animation_list = applet->no_link_images;
-		break;
-	case BUSTED_LINK:
-		animation_list = applet->broken_images;
-		break;
-	case NONE:
-		return;
-	default:
-		g_assert_not_reached ();
-		break;
-	};
-
-	if (num >= g_list_length (animation_list)) {
-		num = 0;
-	}
-
-	image = g_list_nth (animation_list, num);
-	gtk_image_set_from_file (GTK_IMAGE (applet->pixmap),
-			(char*)image->data);
-	num++;
-
-	return TRUE;
-}
-
-static void
-wireless_applet_start_animation (WirelessApplet *applet) 
-{
-	applet->animate_timer = gtk_timeout_add (500, 
-			(GtkFunction)wireless_applet_animate_timeout,
-			applet);
-}
-
-static void
-wireless_applet_stop_animation (WirelessApplet *applet) 
-{
-	if (applet->animate_timer > 0)
-		gtk_timeout_remove (applet->animate_timer);
-	gtk_image_set_from_file (GTK_IMAGE (applet->pixmap),
-			applet->current_pixmap);	
-}
-
-static void
-wireless_applet_animation_state (WirelessApplet *applet) 
-{
-	if (applet->flashing == FALSE) {
-		wireless_applet_start_animation (applet);
-		applet->flashing = TRUE;
+		applet->current_pixbuf = (GdkPixbuf *)applet->pixmaps[state];
+		gtk_image_set_from_pixbuf (GTK_IMAGE (applet->pixmap),
+				applet->current_pixbuf);
 	}
 }
 
@@ -237,7 +186,7 @@ wireless_applet_update_state (WirelessApplet *applet,
 	if (level < 0) {
 		percent = -1;
 	} else {
-		if (link<1) {
+		if (link < 1) {
 			percent = 0;
 		} else {
 			percent = (int)rint ((log (link) / log (92)) * 100.0);
@@ -245,121 +194,25 @@ wireless_applet_update_state (WirelessApplet *applet,
 		}
 	}
 
-	if (percent < 0) {
-		applet->state = BUSTED_LINK;
-		wireless_applet_animation_state (applet);
-	} else if (percent == 0) {
-		applet->state = NO_LINK;
-		wireless_applet_animation_state (applet);
-	} else if (applet->flashing) {
-		applet->state = NONE;
-		wireless_applet_stop_animation (applet);
-	}
-
 	wireless_applet_draw (applet, percent);
 }
 
-static void
-wireless_applet_load_theme_image (WirelessApplet *applet,
-				 const char *themedir,
-				 const char *filename) 
-{
-	int j;
-	char *dot_pos = strrchr (filename, '.') + 1; /* only called if a previous strrchr worked */
-
-	/* Check the allowed extensions */
-	for (j = 0; pixmap_extensions[j]; j++) {
-		if (strcasecmp (dot_pos, pixmap_extensions[j])==0) { 
-			int i;
-			int pixmap_offset_begin = 0;
-			int pixmap_offset_end = 0;
-			char *dupe;
-			gboolean check_range = FALSE;
-
-			if (strncmp (filename, "signal-", 7) == 0) {
-				sscanf (filename, "signal-%d-%d.",
-					&pixmap_offset_begin, &pixmap_offset_end);
-				check_range = TRUE;
-			} else if (strncmp (filename, "no-link-", 8) == 0) {
-				applet->no_link_images = g_list_prepend (applet->no_link_images,
-									     g_strdup_printf ("%s/%s", 
-											      themedir, 
-											      filename));
-			} else if (strncmp (filename, "broken-", 7) == 0) {
-				applet->broken_images = g_list_prepend (applet->broken_images,
-									    g_strdup_printf ("%s/%s", 
-											     themedir, 
-											     filename));
-			}
-
-			if (check_range) {
-				dupe = g_strdup_printf ("%s/%s", themedir, filename);
-				applet->images = g_list_prepend (applet->images,
-								     dupe);
-				for (i = pixmap_offset_begin; i <= pixmap_offset_end; i++) {
-					if (applet->pixmaps[i] != NULL) {
-						show_warning_dialog ("Probable image overlap in\n"
-								"%s.", filename);
-
-					} else {
-						applet->pixmaps[i] = dupe;
-					}
-				}
-			}
-		}
-	}
-}
-
 static void
 wireless_applet_load_theme (WirelessApplet *applet) {
-	DIR *dir;
-	struct dirent *dirent;
 	char *pixmapdir;
+	char *pixmapname;
+	int i;
 
 	pixmapdir = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
 			"wireless-applet/", FALSE, NULL);
-	dir = opendir (pixmapdir);
 
-	/* blank out */
-	if (applet->images) {
-		int j;
-		g_list_foreach (applet->no_link_images, (GFunc)g_free, NULL);
-		g_list_free (applet->no_link_images);
-		applet->no_link_images = NULL;
 
-		g_list_foreach (applet->broken_images, (GFunc)g_free, NULL);
-		g_list_free (applet->broken_images);
-		applet->broken_images = NULL;
-
-		g_list_foreach (applet->images, (GFunc)g_free, NULL);
-		g_list_free (applet->images);
-		applet->images = NULL;
-		for (j=0; j < 101; j++) {
-			applet->pixmaps[j] = NULL;
-		}
-	}
-
-	if (!dir) {
-		show_error_dialog (_("The images necessary to run the wireless monitor are missing.\nPlease make sure that it is correctly installed."));
-	} else 
-		while ((dirent = readdir (dir)) != NULL) {
-			if (*dirent->d_name != '.') {
-				if (strrchr (dirent->d_name, '.')!=NULL) {
-					wireless_applet_load_theme_image (applet, 
-									 pixmapdir,
-									 dirent->d_name);
-				}
-			}
-		}
-
-	if (applet->no_link_images && g_list_length (applet->no_link_images) > 1) {
-		applet->no_link_images = g_list_sort (applet->no_link_images,
-							  (GCompareFunc)g_ascii_strncasecmp);
-	}
-	
-	if (applet->broken_images && g_list_length (applet->broken_images) > 1) {
-		applet->broken_images = g_list_sort (applet->broken_images,
-							 (GCompareFunc)g_ascii_strncasecmp);
+	for (i = 0; i < PIX_NUMBER; i++)
+	{
+		pixmapname = g_build_filename (G_DIR_SEPARATOR_S,
+				pixmapdir, pixmap_names[i], NULL);
+		applet->pixmaps[i] = gdk_pixbuf_new_from_file (pixmapname, NULL);
+		g_free (pixmapname);
 	}
 
 	g_free (pixmapdir);
@@ -382,11 +235,6 @@ wireless_applet_set_show_percent (WirelessApplet *applet, gboolean show) {
 	}
 }
 
-static void
-wireless_applet_set_show_dialogs (WirelessApplet *applet, gboolean show) {
-	applet->show_dialogs = show;
-}
-
 /* check stats, modify the state attribute */
 static void
 wireless_applet_read_device_state (WirelessApplet *applet)
@@ -474,7 +322,7 @@ show_error_dialog (gchar *mesg,...)
 	va_start (ap,mesg);
 	tmp = g_strdup_vprintf (mesg,ap);
 	dialog = gtk_message_dialog_new (NULL,
-			0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+			0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
 			mesg, NULL);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
@@ -492,26 +340,7 @@ show_warning_dialog (gchar *mesg,...)
 	va_start (ap,mesg);
 	tmp = g_strdup_vprintf (mesg,ap);
 	dialog = gtk_message_dialog_new (NULL,
-			0, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
-			mesg, NULL);
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
-	g_free (tmp);
-	va_end (ap);
-}
-
-
-static void
-show_message_dialog (char *mesg,...)
-{
-	GtkWidget *dialog;
-	char *tmp;
-	va_list ap;
-
-	va_start (ap,mesg);
-	tmp = g_strdup_vprintf (mesg,ap);
-	dialog = gtk_message_dialog_new (NULL,
-			0, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+			0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
 			mesg, NULL);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
@@ -543,14 +372,11 @@ wireless_applet_load_properties (WirelessApplet *applet)
 	{
 		applet->device = g_strdup (CFG_DEVICE);
 		applet->show_percent = TRUE;
-		applet->show_dialogs = TRUE;
 		return;
 	}
 
 	applet->show_percent = panel_applet_gconf_get_bool
 		(PANEL_APPLET (applet), "percent", NULL);
-	applet->show_dialogs = panel_applet_gconf_get_bool
-		(PANEL_APPLET (applet), "dialog", NULL);
 }
 
 static void
@@ -561,8 +387,6 @@ wireless_applet_save_properties (WirelessApplet *applet)
 			"device", applet->device, NULL);
 	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
 			"percent", applet->show_percent, NULL);
-	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
-			"dialog", applet->show_dialogs, NULL);
 }
 
 static void
@@ -580,12 +404,6 @@ wireless_applet_option_change (GtkWidget *widget, gpointer user_data)
 			gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON (entry)));
 
-	entry = g_object_get_data (G_OBJECT (applet->prefs),
-			"show-dialog-button");
-	wireless_applet_set_show_dialogs (applet, 
-			gtk_toggle_button_get_active
-			(GTK_TOGGLE_BUTTON (entry)));
-
 	entry = g_object_get_data (G_OBJECT (applet->prefs), "device-menu");
 
 	menu = gtk_menu_get_active
@@ -622,6 +440,7 @@ wireless_applet_properties_dialog (BonoboUIComponent *uic,
 	}
 
 	applet->prefs = glade_property_box;
+	gtk_window_set_resizable (GTK_WINDOW (applet->prefs), FALSE);
 
 	pct = glade_xml_get_widget (xml, "pct_check_button");
 	dialog = glade_xml_get_widget (xml, "dialog_check_button");
@@ -637,16 +456,6 @@ wireless_applet_properties_dialog (BonoboUIComponent *uic,
 	gtk_object_set_data (GTK_OBJECT (applet->prefs),
 			"show-percent-button", pct);
 
-	/* Set the show-dialog thingy */
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog),
-			applet->show_dialogs);
-	g_signal_connect (GTK_OBJECT (dialog),
-			"toggled",
-			GTK_SIGNAL_FUNC (wireless_applet_option_change),
-			applet);
-	gtk_object_set_data (GTK_OBJECT (applet->prefs),
-			"show-dialog-button", dialog);
-
         /* Set the device menu */
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU (device));
 	{
@@ -677,6 +486,20 @@ wireless_applet_properties_dialog (BonoboUIComponent *uic,
 			}
 			idx++;
 		}
+		if (applet->devices == NULL) {
+			char *markup;
+			GtkWidget *label;
+			
+			label = gtk_label_new (NULL);
+			markup = g_strdup_printf ("<i>%s</i>",
+					_("No Wireless Devices"));
+			gtk_label_set_markup (GTK_LABEL (label), markup);
+			g_free (markup);
+
+			item = gtk_menu_item_new ();
+			gtk_container_add (GTK_CONTAINER (item), label);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		}
 		gtk_option_menu_set_menu (GTK_OPTION_MENU (device), menu);
 		gtk_option_menu_set_history (GTK_OPTION_MENU (device), choice);
 	}
@@ -684,7 +507,7 @@ wireless_applet_properties_dialog (BonoboUIComponent *uic,
 
 	g_signal_connect (GTK_OBJECT (applet->prefs),
 			"response", 
-			GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			G_CALLBACK (prefs_response_cb),
 			NULL);
 	g_signal_connect (GTK_OBJECT (applet->prefs),
 			"destroy",
@@ -693,10 +516,35 @@ wireless_applet_properties_dialog (BonoboUIComponent *uic,
 
 	g_object_add_weak_pointer (G_OBJECT (applet->prefs),
 			(void**)&(applet->prefs));
-
+	gtk_window_set_screen (GTK_WINDOW (applet->prefs),
+			       gtk_widget_get_screen (GTK_WIDGET (applet)));
 	gtk_widget_show_all (applet->prefs);
 }
 
+static void
+wireless_applet_help_cb (BonoboUIComponent *uic, WirelessApplet *applet)
+{
+	GError *error = NULL;
+	egg_help_display_on_screen ("wireless", NULL,
+				   gtk_widget_get_screen (GTK_WIDGET (
+						applet)), &error);
+	if (error) {
+		GtkWidget *dialog =
+		gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+					_("There was an error displaying help: %s"),
+					error->message);
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (
+							      applet)));
+		gtk_widget_show (dialog);
+		g_error_free (error);
+	}
+}
+
 static void
 wireless_applet_about_cb (BonoboUIComponent *uic, WirelessApplet *applet)
 {
@@ -743,33 +591,56 @@ wireless_applet_about_cb (BonoboUIComponent *uic, WirelessApplet *applet)
 	return;
 }
 
+static void
+prefs_response_cb (GtkDialog *dialog, gint response, gpointer data)
+{
+	GError *error = NULL;
+	if (response == GTK_RESPONSE_HELP) {
+		egg_help_display_on_screen ("wireless", "wireless-prefs",
+					    gtk_widget_get_screen (GTK_WIDGET (
+					    dialog)), &error);
+		if (error) {
+			GtkWidget *dlg =
+			gtk_message_dialog_new (GTK_WINDOW (dialog),
+						GTK_DIALOG_DESTROY_WITH_PARENT
+						|| GTK_DIALOG_MODAL,
+						GTK_MESSAGE_ERROR,
+						GTK_BUTTONS_CLOSE,
+						_("There was an error displaying help: %s"),
+						error->message);
+			g_signal_connect (G_OBJECT (dlg), "response",
+					  G_CALLBACK (gtk_widget_destroy),
+					  NULL);
+			gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
+			gtk_window_set_screen (GTK_WINDOW (dlg),
+					       gtk_widget_get_screen (
+							GTK_WIDGET (dialog)));
+			gtk_widget_show (dlg);
+			g_error_free (error);
+		}
+	}
+	else
+		gtk_widget_hide (GTK_WIDGET (dialog));
+}
+
 static void
 wireless_applet_destroy (WirelessApplet *applet, gpointer horse)
 {
+	int i;
+
 	g_free (applet->device);
 
-	g_list_foreach (applet->no_link_images, (GFunc)g_free, NULL);
-	g_list_free (applet->no_link_images);
-
-	g_list_foreach (applet->broken_images, (GFunc)g_free, NULL);
-	g_list_free (applet->broken_images);
-
-	g_list_foreach (applet->images, (GFunc)g_free, NULL);
-	g_list_free (applet->images);
-
 	g_list_foreach (applet->devices, (GFunc)g_free, NULL);
 	g_list_free (applet->devices);
 
-	if (applet->animate_timer > 0) {
-		gtk_timeout_remove (applet->animate_timer);
-		applet->animate_timer = 0;
-	}
-
 	if (applet->timeout_handler_id > 0) {
 		gtk_timeout_remove (applet->timeout_handler_id);
 		applet->timeout_handler_id = 0;
 	}
 
+	for (i = 0; i < PIX_NUMBER; i++)
+		g_object_unref (applet->pixmaps[i]);
+
 	if (applet->prefs) {
 		gtk_widget_destroy (applet->prefs);
 		applet->prefs = NULL;
@@ -777,38 +648,103 @@ wireless_applet_destroy (WirelessApplet *applet, gpointer horse)
 
 	if (applet->file)
 		fclose (applet->file);
+	if (applet->tips)
+		g_object_unref (applet->tips);
+}
+
+static void
+setup_widgets (WirelessApplet *applet)
+{
+	GtkRequisition req;
+	gint total_size = 0;
+	gboolean horizontal = FALSE;
+	gint panel_size;
+	
+	panel_size = panel_applet_get_size (PANEL_APPLET (applet));
+	
+	switch (panel_applet_get_orient(PANEL_APPLET (applet))) {
+	case PANEL_APPLET_ORIENT_LEFT:
+	case PANEL_APPLET_ORIENT_RIGHT:
+		horizontal = FALSE;
+		break;
+	case PANEL_APPLET_ORIENT_UP:
+	case PANEL_APPLET_ORIENT_DOWN:
+		horizontal = TRUE;
+		break;
+	}
+
+	/* construct pixmap widget */
+	applet->pixmap = gtk_image_new ();
+	gtk_image_set_from_pixbuf (GTK_IMAGE (applet->pixmap), applet->pixmaps[PIX_BROKEN]);
+	gtk_widget_size_request (applet->pixmap, &req);
+	gtk_widget_show (applet->pixmap);
+
+	if (horizontal)
+		total_size += req.height;
+	else
+		total_size += req.width;
+
+	/* construct pct widget */
+	applet->pct_label = gtk_label_new ("N/A");
+
+	if (applet->show_percent == TRUE) {
+		gtk_widget_show (applet->pct_label);
+		gtk_widget_size_request (applet->pct_label, &req);
+		if (horizontal)
+			total_size += req.height;
+		else
+			total_size += req.width;
+	}
+
+	/* pack */
+	if (applet->box)
+		gtk_widget_destroy (applet->box);
+
+	if (horizontal && (total_size <= panel_size))
+		applet->box = gtk_vbox_new (FALSE, 0);
+	else if (horizontal && (total_size > panel_size))
+		applet->box = gtk_hbox_new (FALSE, 0);
+	else if (!horizontal && (total_size <= panel_size))
+		applet->box = gtk_hbox_new (FALSE, 0);
+	else 
+		applet->box = gtk_vbox_new (FALSE, 0);
+	
+	gtk_box_pack_start (GTK_BOX (applet->box), applet->pixmap, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (applet->box), applet->pct_label, TRUE, TRUE, 0);
+	/* note, I don't use show_all, because this way the percent label is
+	 * only realised if it's enabled */
+	gtk_widget_show (applet->box);
+	gtk_container_add (GTK_CONTAINER (applet), applet->box);
+
+	applet->current_pixbuf = NULL;
+}
+
+static void change_size_cb(PanelApplet *pa, gint s, WirelessApplet *applet)
+{
+	setup_widgets (applet);
+	wireless_applet_timeout_handler (applet);
+}
+
+static void change_orient_cb(PanelApplet *pa, gint s, WirelessApplet *applet)
+{
+	setup_widgets (applet);
+	wireless_applet_timeout_handler (applet);
 }
 
 static GtkWidget *
 wireless_applet_new (WirelessApplet *applet)
 {
-	GtkWidget *box;
+	panel_applet_set_flags (PANEL_APPLET (applet), PANEL_APPLET_EXPAND_MINOR);
 
 	/* this ensures that properties are loaded */
 	wireless_applet_load_properties (applet);
 	wireless_applet_load_theme (applet);
 
-	/* construct pixmap widget */
-	applet->pixmap = gtk_image_new_from_file (applet->pixmaps[0]);
-	gtk_widget_show (applet->pixmap);
-
-	/* construct pct widget */
-	applet->pct_label = gtk_label_new (NULL);
-
-	if (applet->show_percent == TRUE) {
-		gtk_widget_show (applet->pct_label);
-	}
-
-	/* pack */
-	box = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), applet->pixmap, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), applet->pct_label, FALSE, FALSE, 0);
-	/* note, I don't use show_all, because this way the percent label is
-	 * only realised if it's enabled */
-	gtk_widget_show (box);
-	gtk_container_add (GTK_CONTAINER (applet), box);
+	setup_widgets (applet);
 
 	applet->tips = gtk_tooltips_new ();
+	g_object_ref (applet->tips);
+	gtk_object_sink (GTK_OBJECT (applet->tips));
 	applet->prefs = NULL;
 
 	g_signal_connect (GTK_OBJECT (applet),"destroy",
@@ -829,6 +765,11 @@ wireless_applet_new (WirelessApplet *applet)
 		(CFG_UPDATE_INTERVAL * 1000,
 		 (GtkFunction)wireless_applet_timeout_handler,
 		 applet);
+		 
+	g_signal_connect (G_OBJECT (applet), "change_size",
+				  G_CALLBACK (change_size_cb), applet);
+	g_signal_connect (G_OBJECT (applet), "change_orient",
+				  G_CALLBACK (change_orient_cb), applet);
   
 	return GTK_WIDGET (applet);
 }
@@ -865,7 +806,7 @@ wireless_applet_factory (WirelessApplet *applet,
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_Panel_WirelessApplet_Factory",
 		wireless_applet_get_type (),
-		"Wireless applet",
+		"wireless",
 		"0",
 		(PanelAppletFactoryCallback) wireless_applet_factory,
 		NULL)
